
// Sprite.java
// Andrew Davison, April 2005, ad@fivedots.coe.psu.ac.th

/* A Sprite has a position, velocity (in terms of steps),
   an image, and can be deactivated.

   The sprite's image is managed with an ImagesLoader object,
   and an ImagesPlayer object for looping.

   The images stored until the image 'name' can be looped
   through by calling loopImage(), which uses an
   ImagesPlayer object.

*/

import java.awt.*;
import javax.swing.*;
import java.awt.image.*;


public class Sprite {
  // default shpejtsia e sprajtit
  private static final int XSTEP = 5; 
  private static final int YSTEP = 5;

  // default dimensionet kur nuk ka imazh
  private static final int SIZE = 12;   

  // imazhimi
  private ImagesLoader imsLoader;
  private String imageName;
  private BufferedImage image;
  private int width, height;     // dimensionet e imazhit

  private ImagesPlayer player;  // pr t luajtur cikl imazhesh
  private boolean isLooping;

  private int pWidth, pHeight;   // dimensionet e panelit

  private boolean isActive = true;      
  // nj sprajt azhornohet dhe vizatohet vetm kur sht aktiv

  // fusha protected
  protected int locx, locy;        // lokacioni i sprajtit
  protected int dx, dy;            // shpejtsia e lvizjes



  public Sprite(int x, int y, int w, int h, ImagesLoader imsLd, String name) { 
    locx = x; locy = y;
    pWidth = w; pHeight = h;
    dx = XSTEP; dy = YSTEP;

    imsLoader = imsLd;
    setImage(name);    // imazhi default i sprajtit
  }


  /* I shoqron sprajtit emrin e imazhit */
  public void setImage(String name) {
    imageName = name;
    image = imsLoader.getImage(imageName);
    if (image == null) {  // nuk sht gjetur imazh me at emr
      System.out.println("No sprite image for " + imageName);
      width = SIZE;
      height = SIZE;
    } else {
      width = image.getWidth();
      height = image.getHeight();
    }
    // nuk luhet cill imazhi 
    player = null;
    isLooping = false;
  }


  /* Switch on loop playing. The total time for the loop is
  seqDuration secs. The update interval (from the enclosing
  panel) is animPeriod ms. */
  public void loopImage(int animPeriod, double seqDuration) {
    if (imsLoader.numImages(imageName) > 1) {
      player = null;   // pr t inkurajuar garbage collection
      player = new ImagesPlayer(imageName, animPeriod, seqDuration,
                                       true, imsLoader);
      isLooping = true;
    }
    else
      System.out.println(imageName + " is not a sequence of images");
  }


  public void stopLooping() {
    if (isLooping) {
      player.stop();
      isLooping = false;
    }
  }


  public int getWidth()    // of the sprite's image
  {  return width;  }

  public int getHeight()   // of the sprite's image
  {  return height;  }

  public int getPWidth()   // of the enclosing panel
  {  return pWidth;  }

  public int getPHeight()  // of the enclosing panel
  {  return pHeight;  }


  public boolean isActive() 
  {  return isActive;  }

  public void setActive(boolean a) 
  {  isActive = a;  }

  public void setPosition(int x, int y)
  {  locx = x; locy = y;  }

  public void translate(int xDist, int yDist)
  {  locx += xDist;  locy += yDist;  }

  public int getXPosn()
  {  return locx;  }

  public int getYPosn()
  {  return locy;  }


  public void setStep(int dx, int dy)
  {  this.dx = dx; this.dy = dy; }

  public int getXStep()
  {  return dx;  }

  public int getYStep()
  {  return dy;  }


  public Rectangle getMyRectangle() {
	  return  new Rectangle(locx, locy, width, height);
  }


  /* Lviz sprajtin */
  public void updateSprite() {
    if (isActive()) {
      locx += dx;
      locy += dy;
      if (isLooping)
        player.updateTick();  // azhorno player-in
    }
  }



  public void drawSprite(Graphics g) {
    if (isActive()) {
      if (image == null) {   // sprajti nuk ka imazh
        g.setColor(Color.yellow);   // vizato nj rreth t verdh
        g.fillOval(locx, locy, SIZE, SIZE);
        g.setColor(Color.black);
      }
      else {
        if (isLooping)
          image = player.getCurrentImage();
        g.drawImage(image, locx, locy, null);
      }
    }
  }

}
